#include <stdio.h>
#include <stdlib.h>
#include <time.h>

//Fila feita com vetor circular 

struct Fila {
    float *dados;
    int capacidade, tamanho, inicio, fim;
};

struct Fila *cria(int capacidade) { 
    struct Fila *f = (struct Fila*) malloc (sizeof(struct Fila));
	f->capacidade = capacidade;
	f->dados = (float*) malloc (f->capacidade * sizeof(float));
	f->inicio = 0;
	f->fim = -1;
	f->tamanho = 0; 
    return f;
}

int insere(struct Fila *fila, float dado) {
    if(fila->tamanho == fila->capacidade) {
        printf("Fila cheia\n");
        return -1;
    }
    fila->fim = (fila->inicio + fila->tamanho) % fila->capacidade;
    fila->dados[fila->fim] = dado;
    fila->tamanho++;
    return fila->tamanho;
}

float remove_fila(struct Fila *fila) {
    if(fila->tamanho == 0) {
        printf("Fila vazia\n");
        return -1;
    }
    float dado = fila->dados[fila->inicio];
    fila->dados[fila->inicio] = -1;
    fila->inicio = (fila->inicio + 1) % fila->capacidade;
    fila->tamanho--;
    return dado;
}

void imprimir(struct Fila *fila) {
    for(int i = 0; i < fila->capacidade; i++) {
        printf("%.2f ", fila->dados[i]);
    }
    printf("\n");
}

void destroi(struct Fila *fila) {
    free(fila->dados);
    free(fila);
}

// Função para verificar se a fila está vazia
int estaVazia(struct Fila *f) {
    return f->tamanho == 0;
}

// Função para verificar se a fila está cheia
int estaCheia(struct Fila *f) {
    return f->tamanho == f->capacidade;
}


int main() {
/* PRIMEIRA FILA TEMPO */
    struct Fila *fila1 = cria(500);
    struct timespec inicio, fim;
    clock_gettime(CLOCK_MONOTONIC, &inicio);

        // Teste com várias inserções e remoções
    // Vamos fazer 1000000 operações de enfileiramento e desenfileiramento para medir a performance
    for (int i = 0; i < 1000000; i++) {
        if (fila1->tamanho < fila1->capacidade) { // Insere metade das vezes
            insere(fila1, (float) i);
        }
        if (i % 2 == 0 && fila1->tamanho > 0) { // Remove metade das vezes
            remove_fila(fila1);
        }
    }

        // Captura o timestamp após as operações
    clock_gettime(CLOCK_MONOTONIC, &fim);
    double tempo_gasto = (fim.tv_sec - inicio.tv_sec) + (fim.tv_nsec - inicio.tv_nsec) / 1000000000.0;    
    printf("Tempo gasto: %.2f segundos\n", tempo_gasto);
    destroi(fila1);

/* SEGUNDA FILA */
    struct Fila *fila = cria(5);
    insere(fila, 1);
    insere(fila, 2);
    insere(fila, 3);
    printf("REMOVIDO: %.2f\n", remove_fila(fila));
    insere(fila, 4);
    insere(fila, 5);
    insere(fila, 6);
    imprimir(fila);
    printf("REMOVIDO: %.2f\n", remove_fila(fila));
    imprimir(fila);
    insere(fila, 6);
    insere(fila, 7);
    imprimir(fila);

    printf("REMOVIDOS: %.2f %.2f\n", remove_fila(fila), remove_fila(fila));
    imprimir(fila);

    insere(fila, 2);
    insere(fila, 3);
    imprimir(fila);
    insere(fila, 4);
    printf("REMOVIDOS: %.2f %.2f\n", remove_fila(fila), remove_fila(fila));

    insere(fila, 4);
    insere(fila, 5);
    imprimir(fila);

    return 0;
}	